from __future__ import unicode_literals

import html2text
from django.db import models
from simple_mail.settings import sm_settings
from django.template import (Context, Template, loader)
from django.core.mail import EmailMultiAlternatives, get_connection
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ImproperlyConfigured

from premailer import transform


class SimpleMail(models.Model):
    """
    Define an email
    """
    key = models.CharField(verbose_name=_(u"Email"), choices=sm_settings.EMAILS, max_length=20, unique=True)
    subject = models.CharField(max_length=255, verbose_name=_(u"Subject"))
    title = models.CharField(max_length=255, verbose_name=_(u"Title"))
    body = models.TextField(verbose_name=_(u"Content"))
    button_label = models.CharField(verbose_name=_(u"Button label"), max_length=80, blank=True)
    button_link = models.CharField(verbose_name=_(u"Button Link"), max_length=255, blank=True)

    created_at = models.DateTimeField(verbose_name=_(u"Created at"), auto_now_add=True)
    updated_at = models.DateTimeField(verbose_name=_(u"Updated at"), auto_now=True)

    def __unicode__(self):
        return self.get_key_display()

    def render(self, context, template_name):
        context_temp = sm_settings.CONTEXT.copy()
        context_temp.update(context)
        context_temp.update({
            'title': Template(self.title).render(Context(context_temp)),
            'button_label': Template(self.button_label).render(Context(context_temp)),
            'button_link': Template(self.button_link).render(Context(context_temp)),
            'body': Template(self.body).render(Context(context_temp)),
        })
        if template_name is None:
            try:
                template_name = sm_settings.TEMPLATE
            except AttributeError:
                raise ImproperlyConfigured('TEMPLATE Should be configured')
        html = loader.render_to_string(template_name, context_temp)
        html = transform(html, base_url=sm_settings.BASE_URL)
        h = html2text.HTML2Text()
        h.ignore_images = True
        h.ignore_tables = True
        text = h.handle(html)
        response = {
            'subject': Template(self.subject).render(Context(context_temp)),
            'message': text,
            'html_message': html,
        }
        return response

    def get_email_message(self, to, context={}, template_name=None, from_email=None, bcc=[],
                          connection=None, attachments=[], headers={}, cc=[], reply_to=[]):
        if from_email is None:
            try:
                from_email = sm_settings.FROM_EMAIL
            except AttributeError:
                raise ImproperlyConfigured('FROM_EMAIL Should be configured')
        email_kwargs = self.render(context, template_name)
        email_message = EmailMultiAlternatives(
            subject=email_kwargs.get('subject'),
            body=email_kwargs.get('message'),
            from_email=from_email,
            to=to,
            bcc=bcc,
            connection=connection,
            attachments=attachments,
            headers=headers,
            cc=cc,
            reply_to=reply_to
        )
        email_message.attach_alternative(html_content, "text/html")
        return email_message

    def send_mail(self, *args, **kwargs):
        """
        Create an alias to the `send` method to be consistent with django base naming
        """
        return self.send(*args, **kwargs)

    def send(self, to, context={}, template_name=None, from_email=None, bcc=[],
             connection=None, attachments=[], headers={}, cc=[], reply_to=[], fail_silently=False):
        """
        Send the email with the template corresponding to `template_name`
        """
        email_message = self.get_email_message(to, context, template_name, from_email, bcc, connection, attachments, headers, cc, reply_to)
        return email_message.send(fail_silently=fail_silently)

    def send_mass_mail(self, to, context={}, template_name=None, from_email=None, bcc=[],
                       connection=None, attachments=[], headers={}, cc=[], reply_to=[],
                       fail_silently=False, auth_user=None, auth_password=None):
        """
        Send the same email with the template corresponding to `template_name` to each recipient in `to`independently
        """
        connection = connection or get_connection(
            username=auth_user,
            password=auth_password,
            fail_silently=fail_silently
        )
        messages = [
            self.get_email_message([recipient], context, template_name, from_email, bcc, connection, attachments, headers, cc, reply_to)
            for recipient in to
        ]
        return connection.send_messages(messages)




from django.contrib import admin
from simple_mail.models import SimpleMail
from django.contrib import admin


class SimpleMailAdmin(admin.ModelAdmin):
    '''
        Admin View for Mail
    '''
    list_display = ('key',)

admin.site.register(SimpleMail, SimpleMailAdmin)


from simple_mail.models import SimpleMail

def send_mail(key, to, context={}, template_name=None, from_email=None, bcc=[],
             connection=None, attachments=[], headers={}, cc=[], reply_to=[], fail_silently=False):
    mail = SimpleMail.objects.get(key=key)
    return mail.send_mail(to, context, template_name, from_email, bcc,
                         connection, attachments, headers, cc, reply_to, fail_silently)



"""
Settings for PDF Generator are all namespaced in the SIMPLE_MAIL setting.
For example your project's `settings.py` file might look like this:

SIMPLE_MAIL = {
    'CONTEXT': {
        'company_name': '',
        'footer_links': [],
        'footer_content': '',
        'header_url': 'http://placehold.it/600x150',
        'footer_content': "Here is your content",
        'colors': {
            'background': "#DEE0E2",
            'container_border': "#BBBBBB",
            'container_background': "#F4F4F4",
            'container': "#505050",
            'title': "#202020",
            'footer': "#808080",
        }
    },
    'TEMPLATE': 'simple_mail/default.html'
}

This module provides the `sm_settings` object, that is used to access
Simple Mail settings, checking for user settings first, then falling
back to the defaults.
"""
from __future__ import unicode_literals
import os
import collections
import six
from django.conf import settings
from django.test.signals import setting_changed



def dict_nested_update(d, u):
    """
    update values of nested dictionaries keys
    taken from http://stackoverflow.com/questions/3232943/update-value-of-a-nested-dictionary-of-varying-depth
    """
    for k, v in six.iteritems(u):
        if isinstance(v, collections.Mapping):
            r = dict_nested_update(d.get(k, {}), v)
            d[k] = r
        else:
            d[k] = u[k]
    return d


DEFAULTS = {
    'CONTEXT': {
        'header_url': 'http://placehold.it/600x150',
        'footer_links': [],
        'footer_copyright': '',
        'footer_content': "Here is your content",
        'colors': {
            'background': "#EEEEEE",
            'container_border': "#DDDDDD",
            'container_background': "#FFFFFF",
            'container': "#505050",
            'title': "#444444",
            'footer': "#888888",
            'footer_link': "#2C9AB7",
            'button': "#2C9AB7",
        }
    },
    'TEMPLATE': 'simple_mail/default.html',
    'EMAIL_TO': '',
    'EMAILS': [],
    'BASE_URL': '',
    'FROM_EMAIL': ''
}


class SimpleMailSettings(object):
    """
    A settings object, that allows SIMPLE MAIL settings to be accessed as properties.
    For example:
        from simple_mail.settings import sm_settings
        print(sm_settings.TEMPLATE)
    """
    def __init__(self, user_settings=None, defaults=None):
        if user_settings:
            self._user_settings = user_settings
        self.defaults = defaults or DEFAULTS

    @property
    def user_settings(self):
        if not hasattr(self, '_user_settings'):
            self._user_settings = getattr(settings, 'SIMPLE_MAIL', {})
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid Simple Mail setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]
        else:
            # Since all keys of CONTEXT are mandatory
            # Update the user provided value with
            # potential missing keys
            if attr == 'CONTEXT':
                val = dict_nested_update(self.defaults[attr].copy(), val)
        # Cache the result
        setattr(self, attr, val)
        return val


sm_settings = SimpleMailSettings(None, DEFAULTS)


def reload_sm_settings(*args, **kwargs):
    global sm_settings
    setting, value = kwargs['setting'], kwargs['value']
    if setting == 'SIMPLE_MAIL':
        sm_settings = SimpleMailSettings(value, DEFAULTS)


setting_changed.connect(reload_sm_settings)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SimpleMail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(choices=[[b'SUBSCRIPTION', b"Utilisateur - Demande d'inscription"], [b'WELCOME', b'Utilisateur - Bienvenu chez mailedit'], [b'RESET_PASSWORD', b'Utilisateur - R\xc3\xa9initialisation de mot de passe'], [b'USER_PROJECT_SUBM', b'Utilisateur - Votre demande de devis sur mailedit'], [b'ADMIN_PROJECT_SUBM', b'Administrateur - Nouvelle demande de devis re\xc3\xa7ue'], [b'USER_PROJECT_QUOTE', b'Utilisateur - Votre devis sur mailedit'], [b'USER_PROJECT_ORDER', b'Utilisateur - Confirmation de votre commande sur mailedit'], [b'ADMIN_PROJECT_ORDER', b'Administrateur - Nouvelle commande'], [b'USER_PROJECT_PROD', b'Utilisateur - Votre commande est mise en production sur mailedit'], [b'USER_PROJECT_DELVRY', b'Utilisateur - Votre commande a \xc3\xa9t\xc3\xa9 exp\xc3\xa9di\xc3\xa9e'], [b'USER_PROJECT_BILL', b'Utilisateur - Une nouvelle facture a \xc3\xa9t\xc3\xa9 \xc3\xa9mise'], [b'ADMIN_PROJECT_BILL', b'Administrateur - Une nouvelle facture a \xc3\xa9t\xc3\xa9 \xc3\xa9mise']], max_length=20, unique=True, verbose_name='Email')),
                ('subject', models.CharField(max_length=255, verbose_name='Subject')),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('body', models.TextField(verbose_name='Content')),
                ('button_label', models.CharField(blank=True, max_length=80, verbose_name='Button label')),
                ('button_link', models.CharField(blank=True, max_length=255, verbose_name='Button Link')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
            ],
        ),
    ]



#!/usr/bin/env python
import os
from setuptools import setup, find_packages


README = os.path.join(os.path.dirname(__file__), 'README.rst')

# When running tests using tox, README.md is not found
try:
    with open(README) as file:
        long_description = file.read()
except Exception:
    long_description = ''


setup(
    name='django_simple_mail',
    version='1.0.0',
    description='A simple and customizable email template built for Django',
    long_description=long_description,
    url='https://github.com/charlesthk/django-simple-mail',
    author='Charles TISSIER',
    author_email='charles@vingtcinq.io',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.9',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.3',
        'Programming Language :: Python :: 2.4',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='python django mail html template',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'html2text>=2018.1.9',
        'premailer>=3.2.0',
        'six>=1.11.0',
    ],
    # test_suite='tests',
)
